@@ -27,22 +27,22 @@ extensions = ['sphinx.ext.autodoc']
 templates_path = ['.templates']
 
 # The suffix of source filenames.
-source_suffix = '.txt'
+source_suffix = '.rst'
 
 # The master toctree document.
 master_doc = 'index'
 
 # General substitutions.
 project = 'SymPy'
-copyright = '2008, SymPy Development Team'
+copyright = '2012, SymPy Development Team'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '0.5.13'
+version = '0.7.2'
 # The full version, including alpha/beta/rc tags.
-release = '0.5.13'
+release = '0.7.2'
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
