@@ -8,13 +8,13 @@ If you want to derive this by hand, follow the wiki page here:
 
 http://en.wikipedia.org/wiki/Deriving_the_Schwarzschild_solution
 
-Also read the above wiki and follow the references from there if something is
-not clear, like what the Ricci tensor is, etc.
+Also read the above wiki and follow the references from there if
+something is not clear, like what the Ricci tensor is, etc.
 
 """
 
-from sympy import exp, Symbol, sin, Rational, Derivative, dsolve, Function, \
-                  Matrix, Eq, pprint, Pow, classify_ode
+from sympy import (exp, Symbol, sin, Rational, Derivative, dsolve, Function,
+                  Matrix, Eq, pprint, Pow, classify_ode, solve)
 
 def grad(f,X):
     a=[]
@@ -210,10 +210,11 @@ def main():
     #print curvature(Rmn)
     print "-"*40
     print "Solve Einstein's equations:"
-    e = e.subs(nu(r), -lam(r))
+    e = e.subs(nu(r), -lam(r)).doit()
     l =  dsolve(e, lam(r))
     pprint(l)
-    metric = gdd.subs(lam(r), l).subs(nu(r),-l)#.combine()
+    lamsol = solve(l, lam(r))[0]
+    metric = gdd.subs(lam(r), lamsol).subs(nu(r),-lamsol)#.combine()
     print "metric:"
     pprint(metric)
 
