@@ -1,10 +1,12 @@
 #!/usr/bin/env python
+
 """Basic example
 
 Demonstrates how to create symbols and print some algebra operations.
 """
 
 import sympy
+from sympy import pprint
 
 def main():
     a = sympy.Symbol('a')
@@ -12,7 +14,9 @@ def main():
     c = sympy.Symbol('c')
     e = ( a*b*b + 2*b*a*b )**c
 
-    print e
+    print
+    pprint(e)
+    print
 
 if __name__ == "__main__":
     main()
