@@ -1,22 +1,28 @@
 #!/usr/bin/env python
+
 """Functions example
 
 Demonstrates sympy defined functions.
 """
 
 import sympy
+from sympy import pprint
 
 def main():
     a = sympy.Symbol('a')
     b = sympy.Symbol('b')
     e = sympy.log((a + b)**5)
-    print e
+    print
+    pprint(e)
+    print '\n'
 
     e = sympy.exp(e)
-    print e
+    pprint(e)
+    print '\n'
 
     e = sympy.log(sympy.exp((a + b)**5))
-    print e
+    pprint(e)
+    print
 
 if __name__ == "__main__":
     main()
