@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+
 """Vandermonde matrix example
 
 Demonstrates matrix computations using the Vandermonde matrix.
@@ -41,13 +42,16 @@ def vandermonde(order, dim=1, syms='a b c d'):
 
     Define syms to give beginning strings for temporary variables.
 
-    Returns the Matrix, the temporary variables, and the terms for the polynomials
+    Returns the Matrix, the temporary variables, and the terms for the
+    polynomials.
     """
     syms = syms.split()
-    if len(syms) < dim:
+    n = len(syms)
+    if n < dim:
         new_syms = []
-        for i in range(dim - len(syms)):
-            new_syms.append(syms[i%len(syms)] + str(i/len(syms)))
+        for i in range(dim - n):
+            j, rem = divmod(i, n)
+            new_syms.append(syms[rem] + str(j))
         syms.extend(new_syms)
     terms = []
     for i in range(order + 1):
@@ -71,17 +75,16 @@ def gen_poly(points, order, syms):
     """Generates a polynomial using a Vandermonde system"""
     num_pts = len(points)
     if num_pts == 0:
-        raise ValueError, "Must provide points"
+        raise ValueError("Must provide points")
     dim = len(points[0]) - 1
     if dim > len(syms):
-        raise ValueError, \
-            "Must provide at lease %d symbols for the polynomial" % dim
+        raise ValueError("Must provide at lease %d symbols for the polynomial" % dim)
     V, tmp_syms, terms = vandermonde(order, dim)
     if num_pts < V.shape[0]:
-        raise ValueError, \
+        raise ValueError(
             "Must provide %d points for order %d, dimension "\
             "%d polynomial, given %d points" % \
-            (V.shape[0], order, dim, num_pts)
+            (V.shape[0], order, dim, num_pts))
     elif num_pts > V.shape[0]:
         print "gen_poly given %d points but only requires %d, "\
             "continuing using the first %d points" % \
@@ -131,7 +134,7 @@ def main():
 
     print '-'*79
     print "Polynomial fitting with a Vandermonde Matrix:"
-    x,y,z = symbols('x y z')
+    x,y,z = symbols('x,y,z')
 
     points = [(0,3), (1,2), (2,3)]
     print """
