@@ -1,6 +1,18 @@
-from sympy import Symbol, Rational, cos, sin, tan, cot, exp, log
+from sympy import Symbol, Rational, cos, sin, tan, cot, exp, log, Function, \
+                  Derivative, Expr, symbols, pi, I, S
+from sympy.utilities.pytest import raises
+
+def test_diff():
+    x, y = symbols('x, y')
+    assert Rational(1, 3).diff(x) is S.Zero
+    assert I.diff(x) is S.Zero
+    assert pi.diff(x) is S.Zero
+    assert x.diff(x, 0) == x
+    assert (x**2).diff(x, 2, x) == 0
+    assert (x**2).diff(x, y, 0) == 2*x
+    assert (x**2).diff(x, y) == 0
+    raises(ValueError, lambda: x.diff(1, x))
 
-def testdiff():
     a = Symbol("a")
     b = Symbol("b")
     c = Symbol("c")
@@ -21,7 +33,7 @@ def testdiff():
     e = a*b*c
     assert e.diff(c) == a*b
 
-def testdiff2():
+def test_diff2():
     n3 = Rational(3)
     n2 = Rational(2)
     n6 = Rational(6)
@@ -58,7 +70,25 @@ def test_diff3():
     assert e == 2**a*log(Rational(2))**(-1)
     assert e.diff(a) == 2**a
 
+def test_diff_no_eval_derivative():
+    class My(Expr):
+        def __new__(cls, x):
+            return Expr.__new__(cls, x)
+
+    x, y = symbols('x y')
+    # My doesn't have its own _eval_derivative method
+    assert My(x).diff(x).func is Derivative
+    # it doesn't have y so it shouldn't need a method for this case
+    assert My(x).diff(y) == 0
+
 def test_speed():
     # this should return in 0.0s. If it takes forever, it's wrong.
     x = Symbol("x")
     assert x.diff(x, 10**8) == 0
+
+def test_deriv_noncommutative():
+    A = Symbol("A", commutative=False)
+    f = Function("f")
+    x = Symbol("x")
+    assert A*f(x)*A == f(x)*A**2
+    assert A*f(x).diff(x)*A == f(x).diff(x) * A**2
