@@ -1,6 +1,7 @@
 # Tests for var are in their own file, because var pollutes global namespace.
 
-from sympy import Symbol, var, raises
+from sympy import Symbol, var, Function, FunctionClass
+from sympy.utilities.pytest import raises
 
 # make z1 with call-depth = 1
 def make_z1():
@@ -31,23 +32,22 @@ def test_var():
     assert fg == Symbol('fg')
 
     # check return value
-    assert v  == (d, e, fg)
+    assert v  == [d, e, fg]
 
     # see if var() really injects into global namespace
-    raises(NameError, "z1")
+    raises(NameError, lambda: z1)
     make_z1()
     assert z1 == Symbol("z1")
 
-    raises(NameError, "z2")
+    raises(NameError, lambda: z2)
     make_z2()
     assert z2 == Symbol("z2")
 
 def test_var_return():
-    v1 = var('')
+    raises(ValueError, lambda: var(''))
     v2 = var('q')
     v3 = var('q p')
 
-    assert v1 == None
     assert v2 == Symbol('q')
     assert v3 == (Symbol('q'), Symbol('p'))
 
@@ -62,3 +62,13 @@ def test_var_accepts_comma():
 def test_var_keywords():
     var('x y', real=True)
     assert x.is_real and y.is_real
+
+def test_var_cls():
+    f = var('f', cls=Function)
+
+    assert isinstance(f, FunctionClass)
+
+    g, h = var('g,h', cls=Function)
+
+    assert isinstance(g, FunctionClass)
+    assert isinstance(h, FunctionClass)
