@@ -1,5 +1,6 @@
-from sympy import Ylm, Zlm, Symbol, sympify, sqrt, pi, sin, cos, exp, I
+from sympy import Ylm, Zlm, Symbol, sympify, sqrt, pi, sin, cos, exp, I, S
 from sympy.functions.special.spherical_harmonics import Pl, Plm, Plmcos
+from sympy.utilities.pytest import XFAIL
 
 def test_Pl():
     x = Symbol("x")
@@ -43,7 +44,7 @@ def test_Plmcos():
     assert Plmcos(2, 1, th) == -3*cos(th)*sin(th)
     assert Plmcos(2, 2, th) in [3*sin(th)**2, 3*(1-cos(th)**2)]
     assert Plmcos(3, 0, th) == (5*cos(th)**3-3*cos(th))/2
-    assert Plmcos(3, 1, th) == -3*(5*cos(th)**2-1)/2 *sin(th)
+    assert Plmcos(3, 1, th) == -sin(th)*(15*cos(th)**2/2-S(3)/2)
     assert Plmcos(3, 2, th) == 15*cos(th)*sin(th)**2
     assert Plmcos(3, 3, th) == -15*sin(th)**3
 
@@ -56,16 +57,15 @@ def test_Ylm():
     assert Ylm(1, 0, th, ph) == sympify(1)/2 * sqrt(3/pi) * cos(th)
     assert Ylm(1, 1, th, ph) == -sympify(1)/2 * sqrt(3/(2*pi)) * sin(th) * \
             exp(I*ph)
-    #Ylm returns here a correct, but different expression:
-    #assert Ylm(2, -2, th, ph).expand() == (sympify(1)/4 * sqrt(15/(2*pi)) * \
-    #        sin(th)**2 * exp(-2*I*ph)).expand()
     assert Ylm(2, 0, th, ph).expand() == (sympify(1)/4 * sqrt(5/pi) * \
             (3*cos(th)**2-1)).expand()
     assert Ylm(2, 1, th, ph).expand() == (-sympify(1)/2 * \
             sqrt(3)*sqrt(5/(2*pi)) * (sin(th)*cos(th)) * exp(I*ph)).expand()
-    #Ylm returns here a correct, but different expression:
-    #assert Ylm(2, 2, th, ph).expand() == (sympify(1)/4 * sqrt(15/(2*pi)) * \
-    #        sin(th)**2 * exp(2*I*ph)).expand()
+
+    # These last 2 return the correct answer, but the answer can be simplified
+    assert Ylm(2, -2, th, ph).expand() == -sqrt(30)*exp(-2*I*ph)*cos(th)**S(2)/(8*sqrt(pi)) + \
+            S(sqrt(30)*exp(-2*I*ph))/(8*sqrt(pi))
+    assert Ylm(2, 2, th, ph).expand() == S(-sqrt(30)*exp(2*I*ph)*cos(th)**2)/(8*sqrt(pi)) + S(sqrt(30)*exp(2*I*ph))/(8*sqrt(pi))
 
 def test_Zlm():
     #http://en.wikipedia.org/wiki/Solid_harmonics#List_of_lowest_functions
