@@ -1,5 +1,5 @@
 from sympy import print_gtk, sin
-from sympy.utilities.pytest import XFAIL
+from sympy.utilities.pytest import XFAIL, raises
 
 # this test fails if python-libxml2 isn't installed. We don't want to depend on
 # anything with SymPy
@@ -8,3 +8,7 @@ def test_1():
     from sympy.abc import x
     print_gtk(x**2, start_viewer=False)
     print_gtk(x**2+sin(x)/4, start_viewer=False)
+
+def test_settings():
+    from sympy.abc import x
+    raises(TypeError, lambda: print_gtk(x, method="garbage"))
