@@ -1,12 +1,17 @@
-from sympy import diff, Integral, Limit, sin, Symbol, Integer, Rational, cos, tan, asin, acos, atan, sinh, cosh, tanh, asinh, acosh, atanh, E, I, oo, pi, GoldenRatio, EulerGamma
+from sympy import diff, Integral, Limit, sin, Symbol, Integer, Rational, cos, \
+    tan, asin, acos, atan, sinh, cosh, tanh, asinh, acosh, atanh, E, I, oo, \
+    pi, GoldenRatio, EulerGamma, Sum, Eq, Ne, Ge, Lt, Float
 from sympy.printing.mathml import mathml, MathMLPrinter
 from xml.dom.minidom import parseString
 
+from sympy.utilities.pytest import raises
+
 x = Symbol('x')
+y = Symbol('y')
 mp = MathMLPrinter()
 
 def test_printmethod():
-    pass #TODO
+    assert mp.doprint(1+x) == '<apply><plus/><ci>x</ci><cn>1</cn></apply>'
 
 def test_mathml_core():
     mml_1 = mp._print(1+x)
@@ -37,6 +42,13 @@ def test_mathml_core():
     assert nodes[1].childNodes[0].nodeValue == '2'
     assert nodes[2].childNodes[0].nodeValue == 'x'
 
+    mml = mp._print(Float(1.0,2)*x)
+    assert mml.nodeName == 'apply'
+    nodes = mml.childNodes
+    assert nodes[0].nodeName == 'times'
+    assert nodes[1].childNodes[0].nodeValue == '1.0'
+    assert nodes[2].childNodes[0].nodeValue == 'x'
+
 def test_mathml_functions():
     mml_1 = mp._print(sin(x))
     assert mml_1.nodeName == 'apply'
@@ -67,6 +79,15 @@ def test_mathml_integrals():
     assert mml_1.childNodes[3].nodeName == 'uplimit'
     assert mml_1.childNodes[4].toxml() == mp._print(integrand).toxml()
 
+def test_mathml_sums():
+    summand = x
+    mml_1 = mp._print(Sum(summand, (x, 1, 10)))
+    assert mml_1.childNodes[0].nodeName == 'sum'
+    assert mml_1.childNodes[1].nodeName == 'bvar'
+    assert mml_1.childNodes[2].nodeName == 'lowlimit'
+    assert mml_1.childNodes[3].nodeName == 'uplimit'
+    assert mml_1.childNodes[4].toxml() == mp._print(summand).toxml()
+
 def test_mathml_tuples():
     mml_1 = mp._print([2])
     assert mml_1.nodeName == 'list'
@@ -86,8 +107,7 @@ def test_mathml_add():
     mml = mp._print(x**5 - x**4 + x)
     assert mml.childNodes[0].nodeName == 'plus'
     assert mml.childNodes[1].childNodes[0].nodeName == 'minus'
-    assert mml.childNodes[1].childNodes[1].nodeName == 'ci'
-    assert mml.childNodes[2].childNodes[0].nodeName == 'power'
+    assert mml.childNodes[1].childNodes[1].nodeName == 'apply'
 
 def test_mathml_Rational():
     mml_1 = mp._print(Rational(1,1))
@@ -110,7 +130,7 @@ def test_mathml_constants():
     mml = mp._print(pi)
     assert mml.nodeName == 'pi'
 
-    assert mathml(GoldenRatio) == '<cn>\xcf\x86</cn>'
+    assert mathml(GoldenRatio) == '<cn>&#966;</cn>'
 
     mml = mathml(EulerGamma)
     assert mml == '<eulergamma/>'
@@ -152,11 +172,45 @@ def test_mathml_trig():
     mml = mp._print(acosh(x))
     assert mml.childNodes[0].nodeName == 'arccosh'
 
+def test_mathml_relational():
+    mml_1 = mp._print(Eq(x,1))
+    assert mml_1.nodeName == 'apply'
+    assert mml_1.childNodes[0].nodeName == 'eq'
+    assert mml_1.childNodes[1].nodeName == 'ci'
+    assert mml_1.childNodes[1].childNodes[0].nodeValue == 'x'
+    assert mml_1.childNodes[2].nodeName == 'cn'
+    assert mml_1.childNodes[2].childNodes[0].nodeValue == '1'
+
+    mml_2 = mp._print(Ne(1,x))
+    assert mml_2.nodeName == 'apply'
+    assert mml_2.childNodes[0].nodeName == 'neq'
+    assert mml_2.childNodes[1].nodeName == 'cn'
+    assert mml_2.childNodes[1].childNodes[0].nodeValue == '1'
+    assert mml_2.childNodes[2].nodeName == 'ci'
+    assert mml_2.childNodes[2].childNodes[0].nodeValue == 'x'
+
+    mml_3 = mp._print(Ge(1,x))
+    assert mml_3.nodeName == 'apply'
+    assert mml_3.childNodes[0].nodeName == 'geq'
+    assert mml_3.childNodes[1].nodeName == 'cn'
+    assert mml_3.childNodes[1].childNodes[0].nodeValue == '1'
+    assert mml_3.childNodes[2].nodeName == 'ci'
+    assert mml_3.childNodes[2].childNodes[0].nodeValue == 'x'
+
+    mml_4 = mp._print(Lt(1,x))
+    assert mml_4.nodeName == 'apply'
+    assert mml_4.childNodes[0].nodeName == 'lt'
+    assert mml_4.childNodes[1].nodeName == 'cn'
+    assert mml_4.childNodes[1].childNodes[0].nodeValue == '1'
+    assert mml_4.childNodes[2].nodeName == 'ci'
+    assert mml_4.childNodes[2].childNodes[0].nodeValue == 'x'
+
+
 def test_c2p():
     """This tests some optional routines that depend on libxslt1 (which is optional)"""
     try:
         from sympy.modules.mathml import c2p
-        #assert c2p(f.mathml) == result
+        assert c2p(f.mathml) == result
     except ImportError:
         pass
 
@@ -256,3 +310,98 @@ def test_symbol():
     assert mml.childNodes[0].childNodes[1].childNodes[2].nodeName == 'mml:mi'
     assert mml.childNodes[0].childNodes[1].childNodes[2].childNodes[0].nodeValue == 'a'
     del mml
+
+def test_mathml_greek():
+    mml = mp._print(Symbol('alpha'))
+    assert mml.nodeName == 'ci'
+    assert mml.childNodes[0].nodeValue == u'\u03b1'
+
+    assert mp.doprint(Symbol('alpha')) == '<ci>&#945;</ci>'
+    assert mp.doprint(Symbol('beta')) == '<ci>&#946;</ci>'
+    assert mp.doprint(Symbol('gamma')) == '<ci>&#947;</ci>'
+    assert mp.doprint(Symbol('delta')) == '<ci>&#948;</ci>'
+    assert mp.doprint(Symbol('epsilon')) == '<ci>&#949;</ci>'
+    assert mp.doprint(Symbol('zeta')) == '<ci>&#950;</ci>'
+    assert mp.doprint(Symbol('eta')) == '<ci>&#951;</ci>'
+    assert mp.doprint(Symbol('theta')) == '<ci>&#952;</ci>'
+    assert mp.doprint(Symbol('iota')) == '<ci>&#953;</ci>'
+    assert mp.doprint(Symbol('kappa')) == '<ci>&#954;</ci>'
+    assert mp.doprint(Symbol('lambda')) == '<ci>&#955;</ci>'
+    assert mp.doprint(Symbol('mu')) == '<ci>&#956;</ci>'
+    assert mp.doprint(Symbol('nu')) == '<ci>&#957;</ci>'
+    assert mp.doprint(Symbol('xi')) == '<ci>&#958;</ci>'
+    assert mp.doprint(Symbol('omicron')) == '<ci>&#959;</ci>'
+    assert mp.doprint(Symbol('pi')) == '<ci>&#960;</ci>'
+    assert mp.doprint(Symbol('rho')) == '<ci>&#961;</ci>'
+    assert mp.doprint(Symbol('varsigma')) == '<ci>&#962;</ci>'
+    assert mp.doprint(Symbol('sigma')) == '<ci>&#963;</ci>'
+    assert mp.doprint(Symbol('tau')) == '<ci>&#964;</ci>'
+    assert mp.doprint(Symbol('upsilon')) == '<ci>&#965;</ci>'
+    assert mp.doprint(Symbol('phi')) == '<ci>&#966;</ci>'
+    assert mp.doprint(Symbol('chi')) == '<ci>&#967;</ci>'
+    assert mp.doprint(Symbol('psi')) == '<ci>&#968;</ci>'
+    assert mp.doprint(Symbol('omega')) == '<ci>&#969;</ci>'
+
+    assert mp.doprint(Symbol('Alpha')) == '<ci>&#913;</ci>'
+    assert mp.doprint(Symbol('Beta')) == '<ci>&#914;</ci>'
+    assert mp.doprint(Symbol('Gamma')) == '<ci>&#915;</ci>'
+    assert mp.doprint(Symbol('Delta')) == '<ci>&#916;</ci>'
+    assert mp.doprint(Symbol('Epsilon')) == '<ci>&#917;</ci>'
+    assert mp.doprint(Symbol('Zeta')) == '<ci>&#918;</ci>'
+    assert mp.doprint(Symbol('Eta')) == '<ci>&#919;</ci>'
+    assert mp.doprint(Symbol('Theta')) == '<ci>&#920;</ci>'
+    assert mp.doprint(Symbol('Iota')) == '<ci>&#921;</ci>'
+    assert mp.doprint(Symbol('Kappa')) == '<ci>&#922;</ci>'
+    assert mp.doprint(Symbol('Lambda')) == '<ci>&#923;</ci>'
+    assert mp.doprint(Symbol('Mu')) == '<ci>&#924;</ci>'
+    assert mp.doprint(Symbol('Nu')) == '<ci>&#925;</ci>'
+    assert mp.doprint(Symbol('Xi')) == '<ci>&#926;</ci>'
+    assert mp.doprint(Symbol('Omicron')) == '<ci>&#927;</ci>'
+    assert mp.doprint(Symbol('Pi')) == '<ci>&#928;</ci>'
+    assert mp.doprint(Symbol('Rho')) == '<ci>&#929;</ci>'
+    assert mp.doprint(Symbol('Sigma')) == '<ci>&#931;</ci>'
+    assert mp.doprint(Symbol('Tau')) == '<ci>&#932;</ci>'
+    assert mp.doprint(Symbol('Upsilon')) == '<ci>&#933;</ci>'
+    assert mp.doprint(Symbol('Phi')) == '<ci>&#934;</ci>'
+    assert mp.doprint(Symbol('Chi')) == '<ci>&#935;</ci>'
+    assert mp.doprint(Symbol('Psi')) == '<ci>&#936;</ci>'
+    assert mp.doprint(Symbol('Omega')) == '<ci>&#937;</ci>'
+
+def test_mathml_order():
+    expr = x**3 + x**2*y + 3*x*y**3 + y**4
+
+    mp = MathMLPrinter({'order': 'lex'})
+    mml = mp._print(expr)
+
+    assert mml.childNodes[1].childNodes[0].nodeName == 'power'
+    assert mml.childNodes[1].childNodes[1].childNodes[0].data == 'x'
+    assert mml.childNodes[1].childNodes[2].childNodes[0].data == '3'
+
+    assert mml.childNodes[4].childNodes[0].nodeName == 'power'
+    assert mml.childNodes[4].childNodes[1].childNodes[0].data == 'y'
+    assert mml.childNodes[4].childNodes[2].childNodes[0].data == '4'
+
+    mp = MathMLPrinter({'order': 'rev-lex'})
+    mml = mp._print(expr)
+
+    assert mml.childNodes[1].childNodes[0].nodeName == 'power'
+    assert mml.childNodes[1].childNodes[1].childNodes[0].data == 'y'
+    assert mml.childNodes[1].childNodes[2].childNodes[0].data == '4'
+
+    assert mml.childNodes[4].childNodes[0].nodeName == 'power'
+    assert mml.childNodes[4].childNodes[1].childNodes[0].data == 'x'
+    assert mml.childNodes[4].childNodes[2].childNodes[0].data == '3'
+
+def test_settings():
+    raises(TypeError, lambda: mathml(Symbol("x"), method="garbage"))
+
+def test_toprettyxml_hooking():
+    # test that the patch doesn't influence the behavior of the standard library
+    import xml.dom.minidom
+    doc = xml.dom.minidom.parseString("<apply><plus/><ci>x</ci><cn>1</cn></apply>")
+    prettyxml_old = doc.toprettyxml()
+
+    mp.apply_patch()
+    mp.restore_patch()
+
+    assert prettyxml_old == doc.toprettyxml()
